package com.qmth.sop.server.api;

import com.qmth.sop.business.bean.auth.FormPropertyBean;
import com.qmth.sop.business.service.ActivitiConsumerService;
import com.qmth.sop.common.util.JacksonUtil;
import io.swagger.annotations.*;
import org.activiti.engine.FormService;
import org.activiti.engine.RepositoryService;
import org.activiti.engine.TaskService;
import org.activiti.engine.form.FormProperty;
import org.activiti.engine.form.StartFormData;
import org.activiti.engine.form.TaskFormData;
import org.activiti.engine.repository.DeploymentBuilder;
import org.activiti.engine.runtime.ProcessInstance;
import org.activiti.engine.task.Task;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.util.CollectionUtils;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import java.util.*;

@Api(tags = "测试流程表单Controller")
@RestController
@RequestMapping("/activiti/form")
public class ActivitiFromController {
    private final static Logger log = LoggerFactory.getLogger(ActivitiFromController.class);

    @Resource
    private ActivitiConsumerService activitiConsumerService;

    @Resource
    private RepositoryService repositoryService;

    @Resource
    FormService formService;

    @Resource
    TaskService taskService;

    /**
     * 注册流程
     *
     * @return
     */
    @ApiOperation(value = "创建流程接口")
    @RequestMapping(value = "/createDeployment", method = RequestMethod.POST)
    @ApiResponses({@ApiResponse(code = 200, message = "返回信息", response = Object.class)})
    public Map<String, Object> createDeployment() {
        DeploymentBuilder builder = repositoryService.createDeployment();
        builder.addClasspathResource("testform1.bpmn");
        return getFlowStartFormData(builder.deploy().getId());
    }

    /**
     * 获取流程提交表单属性接口
     *
     * @return
     */
    @ApiOperation(value = "获取流程提交表单属性接口")
    @RequestMapping(value = "/getFlowStartFormData", method = RequestMethod.POST)
    @ApiResponses({@ApiResponse(code = 200, message = "返回信息", response = Object.class)})
    public Map<String, Object> getFlowStartFormData(@ApiParam(value = "部署流程id", required = true) @RequestParam String deploymentId) {
        Map<String, Object> map = new HashMap<>();
        String processDefinitionId = activitiConsumerService.findProcessDefinitionIdByDeploymentId(deploymentId);
        StartFormData formData = formService.getStartFormData(processDefinitionId);
        List<FormProperty> formProperties = formData.getFormProperties();
        log.info("start formProperties:{}", JacksonUtil.parseJson(formProperties));
        map.put("FormProperty", new FormPropertyBean(null, null, formProperties));
        map.put("processDefinitionId", processDefinitionId);
        map.put("deploymentId", deploymentId);
        return map;
    }

    /**
     * 获取流程提交表单属性接口
     *
     * @return
     */
    @ApiOperation(value = "获取流程表单属性接口")
    @RequestMapping(value = "/getFlowFormData", method = RequestMethod.POST)
    @ApiResponses({@ApiResponse(code = 200, message = "返回信息", response = Object.class)})
    public Map<String, Object> getFlowFormData(@ApiParam(value = "流程id") @RequestParam(required = false) String flowId,
                                               @ApiParam(value = "流程任务id") @RequestParam(required = false) String taskId) {
        List<Task> taskList = null;
        if (Objects.nonNull(flowId) && !Objects.equals(flowId, "")) {
            taskList = taskService.createTaskQuery().processInstanceId(flowId).list();
        } else if (Objects.nonNull(taskId) && !Objects.equals(taskId, "")) {
            taskList = taskService.createTaskQuery().taskId(taskId).list();
        }
        Map<String, Object> varMap = new HashMap<>();
        if (!CollectionUtils.isEmpty(taskList)) {
            List<FormPropertyBean> formPropertyList = new LinkedList<>();
            for (Task t : taskList) {
                String flowTaskId = t.getId();
                String processInstanceId = t.getProcessInstanceId();
                TaskFormData taskFormData = formService.getTaskFormData(flowTaskId);
                List<FormProperty> formProperties = taskFormData.getFormProperties();
                log.info("taskId:{},flowId:{}", flowTaskId, processInstanceId);
                log.info("task formProperties:{}", JacksonUtil.parseJson(formProperties));
                formPropertyList.add(new FormPropertyBean(processInstanceId, flowTaskId, t.getAssignee(), formProperties));
            }
            varMap.put("formProperties", formPropertyList);
        }
        return varMap;
    }

    /**
     * 启动流程
     *
     * @return
     */
    @ApiOperation(value = "启动流程接口")
    @RequestMapping(value = "/startActivityDemo", method = RequestMethod.POST)
    @ApiResponses({@ApiResponse(code = 200, message = "返回信息", response = Object.class)})
    public Map<String, Object> startActivityDemo(@ApiParam(value = "流程定义id", required = true) @RequestParam String processDefinitionId,
                                                 @ApiParam(value = "审批人id", required = true) @RequestParam String approveId,
                                                 HttpServletRequest request) {
        Map<String, Object> varMap = new HashMap<>();
        Map<String, String> formDataMap = new HashMap<>();
        Map<String, String[]> parameterMap = request.getParameterMap();
        Set<Map.Entry<String, String[]>> entrySet = parameterMap.entrySet();
        for (Map.Entry<String, String[]> entry : entrySet) {
            String key = entry.getKey();
            String[] value = entry.getValue();
            formDataMap.put(key, value[0]);
        }
        varMap.putAll(formDataMap);
        ProcessInstance processInstance = formService.submitStartFormData(processDefinitionId, formDataMap);
        if (Objects.nonNull(processInstance)) {
            List<Task> taskList = taskService.createTaskQuery().processInstanceId(processInstance.getId()).list();
            if (!CollectionUtils.isEmpty(taskList)) {
                List<FormPropertyBean> formPropertyList = new LinkedList<>();
                for (Task t : taskList) {
                    String flowTaskId = t.getId();
                    String processInstanceId = t.getProcessInstanceId();
                    TaskFormData taskFormData = formService.getTaskFormData(flowTaskId);
                    List<FormProperty> formProperties = taskFormData.getFormProperties();
                    log.info("flowTaskId:{},flowId:{}", flowTaskId, processInstanceId);
                    log.info("task formProperties:{}", JacksonUtil.parseJson(formProperties));
                    formPropertyList.add(new FormPropertyBean(processInstanceId, flowTaskId, formProperties));
                }
                varMap.put("formProperties", formPropertyList);
            }
        }
        return varMap;
    }

    /**
     * 获取待办
     *
     * @return
     */
    @ApiOperation(value = "获取待办接口")
    @RequestMapping(value = "/getTaskList", method = RequestMethod.POST)
    @ApiResponses({@ApiResponse(code = 200, message = "返回信息", response = Object.class)})
    public Map<String, Object> getTaskList(@ApiParam(value = "用户id", required = true) @RequestParam String userId, HttpServletRequest request) {
        Map<String, Object> varMap = new HashMap<>();
        List<Task> taskList = taskService.createTaskQuery().taskAssignee(userId).list();
        if (!CollectionUtils.isEmpty(taskList)) {
            List<FormPropertyBean> formPropertyList = new LinkedList<>();
            for (Task t : taskList) {
                String flowTaskId = t.getId();
                String processInstanceId = t.getProcessInstanceId();
                String processDefinitionId = t.getProcessDefinitionId();
                TaskFormData taskFormData = formService.getTaskFormData(flowTaskId);
                List<FormProperty> formProperties = taskFormData.getFormProperties();
                for (FormProperty f : formProperties) {
                    log.info("datePattern:{}", f.getType().getInformation("datePattern"));
                    log.info("values:{}", f.getType().getInformation("values"));
                }
                log.info("用户id:{},taskId:{},flowId:{},processDefinitionId:{}", userId, flowTaskId, processInstanceId, processDefinitionId);
                log.info("task formProperties:{}", JacksonUtil.parseJson(formProperties));
                formPropertyList.add(new FormPropertyBean(processInstanceId, flowTaskId, userId, formProperties));
                varMap.put("formProperties", formPropertyList);
            }
        }
        return varMap;
    }

    /**
     * 提交
     *
     * @param taskId
     * @return
     */
    @ApiOperation(value = "流程提交接口")
    @RequestMapping(value = "/complete", method = RequestMethod.POST)
    @ApiResponses({@ApiResponse(code = 200, message = "返回信息", response = Object.class)})
    public Map<String, Object> complete(@ApiParam(value = "流程taskId", required = true) @RequestParam String taskId, HttpServletRequest request) {
        Task task = taskService.createTaskQuery().taskId(taskId).singleResult();
        Map<String, Object> varMap = new HashMap<>();
        Map<String, String> formDataMap = new HashMap<>();
        Map<String, String[]> parameterMap = request.getParameterMap();
        Set<Map.Entry<String, String[]>> entrySet = parameterMap.entrySet();
        for (Map.Entry<String, String[]> entry : entrySet) {
            String key = entry.getKey();
            String[] value = entry.getValue();
            formDataMap.put(key, value[0]);
        }
        varMap.putAll(formDataMap);
        String flowTaskId = task.getId();
        String processInstanceId = task.getProcessInstanceId();
        String processDefinitionId = task.getProcessDefinitionId();
        formService.submitTaskFormData(flowTaskId, formDataMap);
        List<Task> taskList = taskService.createTaskQuery().processInstanceId(processInstanceId).list();
        if (!CollectionUtils.isEmpty(taskList)) {
            List<FormPropertyBean> formPropertyList = new LinkedList<>();
            for (Task t : taskList) {
                String nextFlowTaskId = t.getId();
                TaskFormData taskFormData = formService.getTaskFormData(nextFlowTaskId);
                List<FormProperty> formProperties = taskFormData.getFormProperties();
                log.info("userId:{},taskId:{},flowId:{},processDefinitionId:{}", t.getAssignee(), nextFlowTaskId, processInstanceId, processDefinitionId);
                log.info("task formProperties:{}", JacksonUtil.parseJson(formProperties));
                formPropertyList.add(new FormPropertyBean(processInstanceId, nextFlowTaskId, formProperties));
                varMap.put("formProperties", formPropertyList);
            }
        }
        return varMap;
    }

    /**
     * 根据流程id直接结束流程
     *
     * @return
     * @paramru
     */
    @ApiOperation(value = "结束流程接口")
    @RequestMapping(value = "/deleteProcessInstance", method = RequestMethod.POST)
    @ApiResponses({@ApiResponse(code = 200, message = "返回信息", response = Object.class)})
    public boolean deleteProcessInstance(String runId) {
        return activitiConsumerService.deleteProcessInstance(runId);
    }
}